@@ -18,7 +18,8 @@
 #define _HPDF_H
 
 #include "hpdf_config.h"
-#include "hpdf_version.h"
+
+#define HPDF_UNUSED(a) ((void)(a))
 
 #ifdef HPDF_DLL_MAKE
 #    define HPDF_EXPORT(A)  __declspec(dllexport) A  __stdcall
@@ -52,6 +53,7 @@
 #include "hpdf_types.h"
 
 typedef void         *HPDF_HANDLE;
+typedef HPDF_HANDLE   HPDF_Boolean;
 typedef HPDF_HANDLE   HPDF_Doc;
 typedef HPDF_HANDLE   HPDF_Page;
 typedef HPDF_HANDLE   HPDF_Pages;
@@ -60,18 +62,22 @@ typedef HPDF_HANDLE   HPDF_Image;
 typedef HPDF_HANDLE   HPDF_Font;
 typedef HPDF_HANDLE   HPDF_Outline;
 typedef HPDF_HANDLE   HPDF_Encoder;
+typedef HPDF_HANDLE   HPDF_3DMeasure;
+typedef HPDF_HANDLE   HPDF_ExData;
 typedef HPDF_HANDLE   HPDF_Destination;
 typedef HPDF_HANDLE   HPDF_XObject;
 typedef HPDF_HANDLE   HPDF_Annotation;
 typedef HPDF_HANDLE   HPDF_ExtGState;
 typedef HPDF_HANDLE   HPDF_FontDef;
 typedef HPDF_HANDLE   HPDF_U3D;
+typedef HPDF_HANDLE   HPDF_JavaScript;
 typedef HPDF_HANDLE   HPDF_Error;
 typedef HPDF_HANDLE   HPDF_MMgr;
 typedef HPDF_HANDLE   HPDF_Dict;
 typedef HPDF_HANDLE   HPDF_EmbeddedFile;
 typedef HPDF_HANDLE   HPDF_OutputIntent;
 typedef HPDF_HANDLE   HPDF_Xref;
+typedef HPDF_HANDLE   HPDF_Shading;
 
 #else
 
@@ -114,6 +120,8 @@ HPDF_SetErrorHandler  (HPDF_Doc            pdf,
 HPDF_EXPORT(void)
 HPDF_Free  (HPDF_Doc  pdf);
 
+HPDF_EXPORT(HPDF_MMgr)
+HPDF_GetDocMMgr  (HPDF_Doc doc);
 
 HPDF_EXPORT(HPDF_STATUS)
 HPDF_NewDoc  (HPDF_Doc  pdf);
@@ -186,6 +194,9 @@ HPDF_GetPageByIndex  (HPDF_Doc    pdf,
 /*---------------------------------------------------------------------------*/
 /*---------------------------------------------------------------------------*/
 
+HPDF_EXPORT(HPDF_MMgr)
+HPDF_GetPageMMgr    (HPDF_Page  page);
+
 HPDF_EXPORT(HPDF_PageLayout)
 HPDF_GetPageLayout  (HPDF_Doc   pdf);
 
@@ -441,13 +452,35 @@ HPDF_EXPORT(HPDF_STATUS)
 HPDF_UseCNTEncodings   (HPDF_Doc   pdf);
 
 
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_UseUTFEncodings   (HPDF_Doc   pdf);
+
+
+/*--------------------------------------------------------------------------*/
+/*----- XObject ------------------------------------------------------------*/
+
+HPDF_EXPORT(HPDF_XObject)
+HPDF_Page_CreateXObjectFromImage    (HPDF_Doc       pdf,
+                                     HPDF_Page      page,
+                                     HPDF_Rect      rect,
+                                     HPDF_Image     image,
+                                     HPDF_BOOL      zoom);
+
+HPDF_EXPORT(HPDF_XObject)
+HPDF_Page_CreateXObjectAsWhiteRect  (HPDF_Doc   pdf,
+                                     HPDF_Page  page,
+                                     HPDF_Rect  rect);
+
 /*--------------------------------------------------------------------------*/
 /*----- annotation ---------------------------------------------------------*/
 
 HPDF_EXPORT(HPDF_Annotation)
 HPDF_Page_Create3DAnnot    (HPDF_Page       page,
 							HPDF_Rect       rect,
-							HPDF_U3D        u3d);
+                            HPDF_BOOL       tb,
+                            HPDF_BOOL       np,
+                            HPDF_U3D        u3d,
+                            HPDF_Image      ap);
 
 HPDF_EXPORT(HPDF_Annotation)
 HPDF_Page_CreateTextAnnot  (HPDF_Page       page,
@@ -466,6 +499,15 @@ HPDF_Page_CreateLineAnnot  (HPDF_Page       page,
 							const char     *text,
 							HPDF_Encoder    encoder);
 
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateWidgetAnnot_WhiteOnlyWhilePrint (HPDF_Doc   pdf,
+                                                 HPDF_Page  page,
+                                                 HPDF_Rect  rect);
+
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateWidgetAnnot (HPDF_Page  page,
+                             HPDF_Rect  rect);
+
 HPDF_EXPORT(HPDF_Annotation)
 HPDF_Page_CreateLinkAnnot  (HPDF_Page          page,
                             HPDF_Rect          rect,
@@ -521,6 +563,12 @@ HPDF_Page_CreateStampAnnot  (	HPDF_Page           page,
 								const char*			text,
 								HPDF_Encoder		encoder);
 
+HPDF_EXPORT(HPDF_Annotation)
+HPDF_Page_CreateProjectionAnnot(HPDF_Page page,
+								HPDF_Rect rect,
+								const char* text,
+								HPDF_Encoder encoder);
+
 HPDF_EXPORT(HPDF_Annotation)
 HPDF_Page_CreateSquareAnnot (HPDF_Page          page,
 							 HPDF_Rect          rect,
@@ -537,6 +585,9 @@ HPDF_EXPORT(HPDF_STATUS)
 HPDF_LinkAnnot_SetHighlightMode  (HPDF_Annotation           annot,
                                   HPDF_AnnotHighlightMode   mode);
 
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_LinkAnnot_SetJavaScript(HPDF_Annotation    annot,
+                             HPDF_JavaScript    javascript);
 
 HPDF_EXPORT(HPDF_STATUS)
 HPDF_LinkAnnot_SetBorderStyle  (HPDF_Annotation  annot,
@@ -606,7 +657,7 @@ HPDF_EXPORT(HPDF_STATUS)
 HPDF_TextMarkupAnnot_SetQuadPoints ( HPDF_Annotation annot, HPDF_Point lb, HPDF_Point rb, HPDF_Point rt, HPDF_Point lt); /* l-left, r-right, b-bottom, t-top positions */
 
 HPDF_EXPORT(HPDF_STATUS)
-HPDF_Annot_Set3DView  ( HPDF_MMgr mmgr, 
+HPDF_Annot_Set3DView  ( HPDF_MMgr mmgr,
 					 	HPDF_Annotation	annot,
 					 	HPDF_Annotation	annot3d,
 					 	HPDF_Dict			view);
@@ -628,8 +679,8 @@ HPDF_EXPORT(HPDF_STATUS)
 HPDF_FreeTextAnnot_SetDefaultStyle (HPDF_Annotation  annot, const char* style);
 
 HPDF_EXPORT(HPDF_STATUS)
-HPDF_LineAnnot_SetPosition (HPDF_Annotation annot, 
-							HPDF_Point startPoint, HPDF_LineAnnotEndingStyle startStyle, 
+HPDF_LineAnnot_SetPosition (HPDF_Annotation annot,
+							HPDF_Point startPoint, HPDF_LineAnnotEndingStyle startStyle,
 							HPDF_Point endPoint, HPDF_LineAnnotEndingStyle endStyle);
 
 HPDF_EXPORT(HPDF_STATUS)
@@ -646,6 +697,66 @@ HPDF_Annotation_SetBorderStyle  (HPDF_Annotation  annot,
                                  HPDF_UINT16      dash_off,
                                  HPDF_UINT16      dash_phase);
 
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_ProjectionAnnot_SetExData(HPDF_Annotation annot, HPDF_ExData exdata);
+
+
+/*--------------------------------------------------------------------------*/
+/*----- 3D Measure ---------------------------------------------------------*/
+HPDF_EXPORT(HPDF_3DMeasure)
+HPDF_Page_Create3DC3DMeasure(HPDF_Page       page,
+					         HPDF_Point3D    firstanchorpoint,
+					         HPDF_Point3D    textanchorpoint
+					         );
+
+HPDF_EXPORT(HPDF_3DMeasure)
+HPDF_Page_CreatePD33DMeasure(HPDF_Page       page,
+					         HPDF_Point3D    annotationPlaneNormal,
+					         HPDF_Point3D    firstAnchorPoint,
+   					         HPDF_Point3D    secondAnchorPoint,
+	        				 HPDF_Point3D    leaderLinesDirection,
+   					         HPDF_Point3D    measurementValuePoint,
+					         HPDF_Point3D    textYDirection,
+					         HPDF_REAL       value,
+							 const char*     unitsString
+  					        );
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DMeasure_SetName(HPDF_3DMeasure measure,
+					   const char* name);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DMeasure_SetColor(HPDF_3DMeasure measure,
+						   HPDF_RGBColor color);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DMeasure_SetTextSize(HPDF_3DMeasure measure,
+							  HPDF_REAL textsize);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DC3DMeasure_SetTextBoxSize(HPDF_3DMeasure measure,
+							 HPDF_INT32 x,
+							 HPDF_INT32 y);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DC3DMeasure_SetText(HPDF_3DMeasure measure,
+						  const char* text,
+						  HPDF_Encoder encoder);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DC3DMeasure_SetProjectionAnotation(HPDF_3DMeasure measure,
+										 HPDF_Annotation projectionanotation);
+
+/*--------------------------------------------------------------------------*/
+/*----- External Data ---------------------------------------------------------*/
+
+HPDF_EXPORT(HPDF_ExData)
+HPDF_Page_Create3DAnnotExData(HPDF_Page       page );
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DAnnotExData_Set3DMeasurement(HPDF_ExData exdata, HPDF_3DMeasure measure);
+
+/*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
 /*----- 3D View ---------------------------------------------------------*/
 
@@ -655,6 +766,10 @@ HPDF_Page_Create3DView    (HPDF_Page       page,
 						   HPDF_Annotation	annot3d,
 						   const char *name);
 
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_3DView_Add3DC3DMeasure(HPDF_Dict       view,
+							HPDF_3DMeasure measure);
+
 /*--------------------------------------------------------------------------*/
 /*----- image data ---------------------------------------------------------*/
 
@@ -686,6 +801,21 @@ HPDF_EXPORT(HPDF_Image)
 HPDF_LoadU3DFromFile (HPDF_Doc      pdf,
                             const char    *filename);
 
+HPDF_EXPORT(HPDF_Image)
+HPDF_LoadU3DFromMem  (HPDF_Doc      pdf,
+               const HPDF_BYTE     *buffer,
+                     HPDF_UINT      size);
+
+HPDF_EXPORT(HPDF_Image)
+HPDF_Image_LoadRaw1BitImageFromMem  (HPDF_Doc           pdf,
+                           const HPDF_BYTE   *buf,
+                          HPDF_UINT          width,
+                          HPDF_UINT          height,
+                          HPDF_UINT          line_width,
+                          HPDF_BOOL          black_is1,
+                          HPDF_BOOL          top_is_first);
+
+
 HPDF_EXPORT(HPDF_Image)
 HPDF_LoadRawImageFromFile  (HPDF_Doc           pdf,
                             const char         *filename,
@@ -848,7 +978,7 @@ HPDF_Font_MeasureText (HPDF_Font          font,
 
 
 /*--------------------------------------------------------------------------*/
-/*----- attachements -------------------------------------------------------*/
+/*----- attachments -------------------------------------------------------*/
 
 HPDF_EXPORT(HPDF_EmbeddedFile)
 HPDF_AttachFile  (HPDF_Doc    pdf,
@@ -1058,9 +1188,9 @@ HPDF_Page_SetMiterLimit  (HPDF_Page  page,
 /* d */
 HPDF_EXPORT(HPDF_STATUS)
 HPDF_Page_SetDash  (HPDF_Page           page,
-                    const HPDF_UINT16  *dash_ptn,
+                    const HPDF_REAL  *dash_ptn,
                     HPDF_UINT           num_param,
-                    HPDF_UINT           phase);
+                    HPDF_REAL           phase);
 
 
 
@@ -1077,6 +1207,11 @@ HPDF_EXPORT(HPDF_STATUS)
 HPDF_Page_SetExtGState  (HPDF_Page        page,
                          HPDF_ExtGState   ext_gstate);
 
+/* sh */
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_Page_SetShading  (HPDF_Page    page,
+                       HPDF_Shading shading);
+
 
 /*--- Special graphic state operator --------------------------------------*/
 
@@ -1356,7 +1491,23 @@ HPDF_Page_SetCMYKStroke  (HPDF_Page  page,
 
 /*--- Shading patterns ---------------------------------------------------*/
 
-/* sh --not implemented yet */
+/* Notes for docs:
+ * - ShadingType must be HPDF_SHADING_FREE_FORM_TRIANGLE_MESH (the only
+ *   defined option...)
+ * - colorSpace must be HPDF_CS_DEVICE_RGB for now.
+ */
+HPDF_EXPORT(HPDF_Shading)
+HPDF_Shading_New  (HPDF_Doc         pdf,
+                   HPDF_ShadingType type,
+                   HPDF_ColorSpace  colorSpace,
+                   HPDF_REAL xMin, HPDF_REAL xMax,
+                   HPDF_REAL yMin, HPDF_REAL yMax);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_Shading_AddVertexRGB(HPDF_Shading shading,
+                          HPDF_Shading_FreeFormTriangleMeshEdgeFlag edgeFlag,
+                          HPDF_REAL x, HPDF_REAL y,
+                          HPDF_UINT8 r, HPDF_UINT8 g, HPDF_UINT8 b);
 
 /*--- In-line images -----------------------------------------------------*/
 
@@ -1371,6 +1522,17 @@ HPDF_EXPORT(HPDF_STATUS)
 HPDF_Page_ExecuteXObject  (HPDF_Page     page,
                            HPDF_XObject  obj);
 
+/*--- Content streams ----------------------------------------------------*/
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_Page_New_Content_Stream  (HPDF_Page page,
+                               HPDF_Dict* new_stream);
+
+HPDF_EXPORT(HPDF_STATUS)
+HPDF_Page_Insert_Shared_Content_Stream  (HPDF_Page page,
+                                         HPDF_Dict shared_stream);
+
+
 /*--- Marked content -----------------------------------------------------*/
 
 /* BMC --not implemented yet */
@@ -1446,14 +1608,14 @@ HPDF_EXPORT(HPDF_OutputIntent)
 HPDF_ICC_LoadIccFromMem (HPDF_Doc   pdf,
                         HPDF_MMgr   mmgr,
                         HPDF_Stream iccdata,
-                        HPDF_Xref   xref, 
+                        HPDF_Xref   xref,
                         int         numcomponent);
 
 HPDF_EXPORT(HPDF_OutputIntent)
 HPDF_LoadIccProfileFromFile  (HPDF_Doc  pdf,
                             const char* icc_file_name,
                                    int  numcomponent);
-                                   
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
